#!/usr/bin/env python3
"""A class that holds a learned classifier, to be used in the GUI.
"""

# https://stackoverflow.com/questions/10592605/save-classifier-to-disk-in-scikit-learn
# https://pythonhosted.org/joblib/generated/joblib.dump.html
# https://docs.python.org/2/library/pickle.html

import numpy as np

## MUST BE SKLEARN 0.19.1
from sklearn.externals import joblib

import core.data_processor as dproc
from  core.defs import *

class Classifier:
    
    def __init__(self):
        self._classifier = []
        self._s = None
        self._data_dict = None
        pass

    def read_python2(self, clsfile, datadictfile):
        """Read the classifier and the data dictionary from separate files"""
        self._classifier = joblib.load(clsfile)        
        self._data_dict  = joblib.load(datadictfile)
        self._s = self._data_dict.pop('s_stencil')

    def read_from_file(self, filename):
        """Read the classifier from file"""
        other = joblib.load(filename)
        self._classifier = other._classifier
        self._s = other._s
        self._data_dict = other._data_dict        
        
    def write_to_file(self, filename):
        """Write the classifier to file"""
        joblib.dump(self, filename, compress=COMPRESS, protocol=PICKLE_PROTOCOL)
        
    def predict(self, polygon):
        """Runs the classifier on a polygon, stores the data in the polygon"""

        if(polygon.orientation == -1):
            polygon.mirror_points(False,False,False,True)
            
        dset, _ = dproc.make_dataset([polygon], s_stencil=self._s,**self._data_dict)
        n_augment = dset.n_augment
        X = dset.XX
        
        y_predict = self._classifier.predict(X)
        y_proba =   self._classifier.predict_proba(X)
        y_proba = -1*y_proba[:,0] + 1*y_proba[:,1] # move to -1 1
        polygon.point_data[Y_PROBA] =  dproc.unaugment_scores(y_proba, np.amax, n_augment)
        polygon.point_data[Y_PREDICT] = dproc.unaugment_scores(y_predict, np.amax, n_augment)


        return polygon
