import matplotlib.pyplot as plt
from core.polygon import Polygon
from core.defs import * 

from PIL import Image, ImageDraw
import sys, pdb, os, subprocess, platform

class Node:
    def __init__(self, x, y, adj, r, g, b):
        self.x = x
        self.y = y
        self.adj = adj
        self.r = r
        self.g = g
        self.b = b
        self.visited = False

    def is_connected_top(self):
        return self.adj & (1) != 0
    def is_connected_top_right(self):
        return self.adj & (1<<1) != 0
    def is_connected_right(self):
        return self.adj & (1<<2) != 0
    def is_connected_bottom_right(self):
        return self.adj & (1<<3) != 0
    def is_connected_bottom(self):
        return self.adj & (1<<4) != 0
    def is_connected_bottom_left(self):
        return self.adj & (1<<5) != 0
    def is_connected_left(self):
        return self.adj & (1<<6) != 0
    def is_connected_top_left(self):
        return self.adj & (1<<7) != 0

    def is_boundary(self, w, h):
        if self.x > 0 and not self.is_connected_left(): return True
        if self.x < (w-1) and not self.is_connected_right(): return True
        if self.y > 0 and not self.is_connected_top(): return True
        if self.y < (h-1) and not self.is_connected_bottom(): return True
        return False

    def neighbors_coords(self):
        return [(self.x-1, self.y-1), 
                 (self.x, self.y-1), 
                 (self.x+1, self.y-1),
                 (self.x+1, self.y),
                 (self.x+1, self.y+1),
                 (self.x, self.y+1),
                 (self.x-1, self.y+1),
                 (self.x-1, self.y)]

    def valid_neighbors_coords(self):
        nhs = []
        if self.is_connected_top_left():
            nhs.append((self.x-1, self.y-1))
        if self.is_connected_top():
            nhs.append((self.x, self.y-1))
        if self.is_connected_top_right():
            nhs.append((self.x+1, self.y-1))
        if self.is_connected_right():
            nhs.append((self.x+1, self.y))
        if self.is_connected_bottom_right():
            nhs.append((self.x+1, self.y+1))
        if self.is_connected_bottom():
            nhs.append((self.x, self.y+1))
        if self.is_connected_bottom_left():
            nhs.append((self.x-1, self.y+1))
        if self.is_connected_left():
            nhs.append((self.x-1, self.y))
        return nhs

class Edge:
    def __init__(self, src, dst):
        self.src = src
        self.dst = dst

def extract(filename, out_path):
    lines = []
    with open(filename, 'r') as f:
        lines = f.readlines()

    w, h = lines[0].split()
    w, h = int(w), int(h)

    if w == 0 or h == 0:
        return

    nodes = []
    i = 1
    for y in range(h):
        for x in range(w):
            adj, r, g, b = lines[i].split()
            adj, r, g, b = int(adj), int(r), int(g), int(b)

            nodes.append(Node(x, y, adj, r, g, b))
            i += 1

    def nid(x, y): return w * y + x;

    # Testing Input
    if False:
        SCALE = 5
        img = Image.new('RGB', (w * SCALE, h * SCALE))
        draw = ImageDraw.Draw(img)
        for node in nodes:
            stack = [node]
            while len(stack) > 0:
                node = stack.pop(0)
                if node.visited:
                    continue
                node.visited = True

                #print(node.x, node.y, node.adj, node.r, node.g, node.b)

                for nh in node.valid_neighbors_coords():
                    if nh[0] < 0 or nh[0] >= w or nh[1] < 0 or nh[1] >= h:
                        continue

                    draw.line((node.x * SCALE, node.y * SCALE, nh[0] * SCALE, nh[1] * SCALE))

                    nh_node = nodes[nid(nh[0], nh[1])]      
                    if not nh_node.visited:
                        stack.append(nh_node)
                #draw.point((node.x * SCALE, node.y * SCALE), fill=(node.r, node.g, node.b, 255))

        img.save('out.png', 'PNG')
        return 


    #print('Finding Regions..')
    blobs = []
    c = 0
    for i in range(len(nodes)):
        stack = [nodes[i]]
        regions = []

        while len(stack) > 0:
            cur_node = stack.pop()

            if cur_node.visited: 
                continue
            cur_node.visited = True

            edges = []
            bnode = cur_node
            direction = -1

            if bnode.x > 0 and not bnode.is_connected_left():
                edges.append(Edge((bnode.x, bnode.y+1), (bnode.x, bnode.y)))
                direction = 0

            elif bnode.y > 0 and not bnode.is_connected_top():
                edges.append(Edge((bnode.x, bnode.y), (bnode.x+1, bnode.y)))
                direction = 1

            elif bnode.x < (w-1) and not bnode.is_connected_right():
                edges.append(Edge((bnode.x+1, bnode.y), (bnode.x+1, bnode.y+1)))
                direction = 2

            elif bnode.y < (h-1) and not bnode.is_connected_bottom():
                edges.append(Edge((bnode.x+1, bnode.y+1), (bnode.x, bnode.y+1)))
                direction = 3

            while len(edges) > 0 and edges[0].src != edges[-1].dst:
                end = edges[-1].dst

                if direction == 0:
                    if bnode.is_connected_top_left():
                        edges.append(Edge(end, (end[0]-1, end[1])))
                        bnode = nodes[nid(bnode.x-1, bnode.y-1)]
                        direction = 3
                    elif bnode.is_connected_top():
                        edges.append(Edge(end, (end[0], end[1]-1)))
                        bnode = nodes[nid(bnode.x, bnode.y-1)]
                        direction = 0
                    else:
                        edges.append(Edge(end, (end[0]+1, end[1])))
                        direction = 1

                elif direction == 1:
                    if bnode.is_connected_top_right():
                        edges.append(Edge(end, (end[0], end[1]-1)))
                        bnode = nodes[nid(bnode.x+1, bnode.y-1)]
                        direction = 0
                    elif bnode.is_connected_right():
                        edges.append(Edge(end, (end[0]+1, end[1])))
                        bnode = nodes[nid(bnode.x+1, bnode.y)]
                        direction = 1
                    else:
                        edges.append(Edge(end, (end[0], end[1]+1)))
                        direction = 2

                elif direction == 2:
                    if bnode.is_connected_bottom_right():
                        edges.append(Edge(end, (end[0]+1, end[1])))
                        bnode = nodes[nid(bnode.x+1, bnode.y+1)]
                        direction = 1
                    elif bnode.is_connected_bottom():
                        edges.append(Edge(end, (end[0], end[1]+1)))
                        bnode = nodes[nid(bnode.x, bnode.y+1)]
                        direction = 2
                    else:
                        edges.append(Edge(end, (end[0]-1, end[1])))
                        direction = 3

                elif direction == 3:
                    if bnode.is_connected_bottom_left():
                        edges.append(Edge(end, (end[0], end[1]+1)))
                        bnode = nodes[nid(bnode.x-1, bnode.y+1)]
                        direction = 2
                    elif bnode.is_connected_left():
                        edges.append(Edge(end, (end[0]-1, end[1])))
                        bnode = nodes[nid(bnode.x-1, bnode.y)]
                        direction = 3
                    else:
                        edges.append(Edge(end, (end[0], end[1]-1)))
                        direction = 0
                else:
                    assert(False)

                bnode.visited = True

            if len(edges) > 0:
                regions.append(edges)

            for nh in cur_node.valid_neighbors_coords():
                if nh[0] < 0 or nh[0] >= w or nh[1] < 0 or nh[1] >= h:
                    continue

                nh_node = nodes[nid(nh[0], nh[1])]      
                if not nh_node.visited:
                    stack.append(nh_node)
        if len(regions) > 0:
            blobs.append((regions, nodes[i]))

    #print('Extracted contours')

    # Testing output
    if False:
        img = Image.new('RGB', (w+1, h+1))
        draw = ImageDraw.Draw(img)
        for blob in blobs:
            for region in blob[0]:
                for edge in region:
                    draw.point((edge.src[0], edge.src[1]))
                    draw.point((edge.src[0], edge.dst[1]))
        img.save(out_path + 'contours.png', 'PNG')
        return

    num_polygons = sum(len(b[0]) for b in blobs)
    #print('Found {} blobs and {} contours'.format(len(blobs), num_polygons))
    #print('Exporting {} poygons'.format(num_polygons))

    def boundaries_name(i): return out_path + str(i) + '.boundary'
    def polygon_name(i): return out_path +  str(i) + '.poly'
    def fig_name(i): return out_path +  str(i) + '.png'
    def colors_name(): return out_path + 'colors.txt';

    colors_ff = open(colors_name(), 'w')

    ii = 0
    for blob in blobs:
        for region in blob[0]:
            with open(boundaries_name(ii), 'w') as ff:
                node = nodes[nid(region[0].src[0], region[0].src[1])]
                colors_ff.write('%d %d %d\n'%(node.r, node.g, node.b))
                for edge in region:
                    ff.write('{} {}\n'.format(edge.src[0], edge.src[1]))
            ii += 1

    colors_ff.close()    

    polygons = []
    for pp in range(num_polygons):
        polygon = Polygon()
        polygon.read_polygon_txt(boundaries_name(pp))
        polygon.write_dot_poly(polygon_name(pp))

        plt.figure(1)
        polygons.append(polygon)
        polygon.plot_nice( plt.gca())
        #print('Saving {}'.format(fig_name(pp)))
        plt.savefig(fig_name(pp))
        plt.clf()


def extract_multicolor(in_path, out_path): 
    graph_path = TEMP_DIR+'pixel_graph.txt'

    if( platform.system() == 'Linux'):
        depixelize_exe = 'Executable not present on this platform'
    else:
        depixelize_exe = PIXELGRAPH_EXE_DIR_WIN

    #print('Executing %s'%(depixelize_exe))
    process = subprocess.Popen([depixelize_exe, in_path, graph_path], stderr=subprocess.STDOUT)
    process.communicate()

    extract(graph_path, out_path)
