#!/usr/bin/evn python3
""" Options for running the C++ fitter and corner remover"""

import subprocess
import os
import glob
import platform

from core.defs import *

option_keys = [
    'name',
    '',
    'CONSTRAINT_EVAL_TYPE',
    'ERROR_COST_TYPE',
    'CORNER_PROJ_TYPE',
    'G1_OVERLAP',
    'GRAPH_TRANSITION_APPROX_TYPE',
    'GRAPH_ALLOW_WEIGHT_REDUCTION',
    'SOFT_CURVE_TRANSITION',
    'HALFSPACE_CONSTRAINT',
    'NORMALIZE_ERROR_COST',
    'REGULARIZE',
    'SIMPLIFY',
    'GLOBAL_ANALYSIS',
    'SHORT_SEGMENTS',
    '',
    'AXIS_ALIGNED_LINE_REWARD',
    'AXIS_ALIGNED_ARC_REWARD',
    'LINE_COST',
    'ARC_COST',
    'CLOTHOID_COST',
    'INFLECTION_COST',
    'G0_COST',
    'G1_COST',
    'ERROR_COST',
    'INTERNAL_PARAMETERS_MARKER',
    'ERROR_THRESHOLD_PRIMFIT',
    'CONSTRAINT_MDIST_LINE',
    'CONSTRAINT_MDIST_ARC',
    'CONSTRAINT_MDIST_CLOTH',
    '',
    'CURVE_ADJUST_DAMPING',
    'CURVE_ADJUST_NUMIT',
    'CURVE_ADJUST_G0_EXTRAW',
    'CURVE_REGULAR_NUMIT',
    'CURVE_REGULAR_WEIGHT',
    '',
    'COMBINE_DAMPING',
    'COMBINE_NUMIT',
    'COMBINE_G0_EXTRAW',
    'COMBINE_CONSTRAINT_WEIGHT',
    'COMBINE_SOFT_TANGENT_WEIGHT'
    '',
    'GRAPH_TRANSITION_APPROX_COEFF',
]
            

def default_options():
    return { 
        'name' : 'CLIPART_1',
        ##
        ##
        'CONSTRAINT_EVAL_TYPE' : 1,  # raytrace, dist, previous woodoo 
        'ERROR_COST_TYPE' : 0, # 0 2 3{"L1", "L1/n", "L2", "L2^2", "L2/n", "Linf"}
        'CORNER_PROJ_TYPE' : 1, #mindist endpoint 
        'G1_OVERLAP' : 1, #1
        'GRAPH_TRANSITION_APPROX_TYPE' : 1, # zero cornu trufit
        'GRAPH_ALLOW_WEIGHT_REDUCTION' : 0,
        'SOFT_CURVE_TRANSITION' : 0,
        'HALFSPACE_CONSTRAINT' : 1,
        'NORMALIZE_ERROR_COST' : 0,
        'REGULARIZE': 1,
        'SIMPLIFY': 1,
        'GLOBAL_ANALYSIS': 1,
        'SHORT_SEGMENTS': 1,
        ##
        ##
        'AXIS_ALIGNED_LINE_REWARD' : 0.,
        'AXIS_ALIGNED_ARC_REWARD' : 0.,
        'LINE_COST' : 1.00,
        'ARC_COST' :  2.00, 
        'CLOTHOID_COST' : 4.00,
        'INFLECTION_COST': 0.00,
        'G0_COST' : 1e8,
        'G1_COST' : 0,
        'ERROR_COST' : 1.0,
        'INTERNAL_PARAMETERS_MARKER' : 0.000000, 
        'ERROR_THRESHOLD_PRIMFIT' : 12.000000, #L2 12
        'CONSTRAINT_MDIST_LINE' : 0.1, #0.15
        'CONSTRAINT_MDIST_ARC' : 0.1,  #0.15
        'CONSTRAINT_MDIST_CLOTH' : 0.1, #0.15
        ##
        'CURVE_ADJUST_DAMPING' : 0.1000, 
        'CURVE_ADJUST_NUMIT' : 2.00000,
        'CURVE_ADJUST_G0_EXTRAW' : 1.000,
        'CURVE_REGULAR_NUMIT' : 0.0,
        'CURVE_REGULAR_WEIGHT' : 40.0,
        ##
        'COMBINE_DAMPING' : 0.020,
        'COMBINE_NUMIT' : 40,
        'COMBINE_G0_EXTRAW' : 1.000,
        'COMBINE_CONSTRAINT_WEIGHT' : 100.0,
        'COMBINE_SOFT_TANGENT_WEIGHT' : -5000.0,
        ##
        'GRAPH_TRANSITION_APPROX_COEFF' : 1,
}

def get_preset(reso):

    option_dict = default_options()
    
    if (reso <= 32):
        option_dict['name'] = 'preset-32'
        option_dict['ERROR_COST'] = 1.0
        option_dict['ERROR_THRESHOLD_PRIMFIT'] = 25
    elif (reso == 64):
        option_dict['name'] = 'preset-64'
        option_dict['ERROR_COST'] = 0.5
        option_dict['ERROR_THRESHOLD_PRIMFIT'] = 50
    elif (reso == 128):
        option_dict['name'] = 'preset-128'
        option_dict['ERROR_COST'] = 0.25
        option_dict['ERROR_THRESHOLD_PRIMFIT'] = 100
    elif (reso == 256):
        option_dict['name'] = 'preset-256'
        option_dict['ERROR_COST'] = 0.125
        option_dict['ERROR_THRESHOLD_PRIMFIT'] = 100
    else:
        assert 0, "preset {} not found".format(reso)

    return option_dict
    
    
def write_option(f, option_dict):
    """ Write the options to the file """

    f.write('[options]\n')
    f.write('[./values]\n\n')

    for key in option_keys:
        if(key == ''):
            f.write('\n')
        else:
            f.write("{} = {} \n".format(key, option_dict[key]))

def write_pot_file(fstr,
                   fitter_in_poly,
                   fitter_out_file,
                   options_dict,
                   threshold,
                   use_proba,
                   multirepo=None):
    
    # assert (corners_from == 'proba') or  (corners_from == 'decis') 
        
    log0_file_name = "{}_log0.txt".format(fitter_out_file)
    log1_file_name = "{}_log1.txt".format(fitter_out_file)
    input_file_name = fitter_in_poly
    svg_file_name = "{}_result".format(fitter_out_file)
    corners_from = None
    if(use_proba): corners_from = 'proba'
    else         : corners_from = 'annot'
        
    fstr.write('draw = 0 \n')
    fstr.write('loop = 0 \n')
    fstr.write('NONE = \'NONE\' \n')
    fstr.write('\n\n')
    
    write_option(fstr, options_dict)
    
    fstr.write('[input] \n') 
    fstr.write('readpoly={} \n'.format(input_file_name) )
    if multirepo is not None:
        fstr.write('multirepo={}\n'.format(multirepo))
    fstr.write('\n')

    fstr.write('[log] \n') 
    fstr.write('normal = {} \n'.format(log0_file_name))
    fstr.write('priority = {} \n'.format(log1_file_name))
    fstr.write('\n')

    fstr.write('[graphconstruction] \n') 
    fstr.write('shortest_type = cycle \n')
    fstr.write('\n')

    fstr.write('[globalfit]\n')
    fstr.write('numfit = 30 \n'  )       
    fstr.write('\n')

    fstr.write('[]\n')
    fstr.write('writesvg = {} \n'.format(svg_file_name) )
    fstr.write('\n')

    fstr.write('threshold={:f} \n'.format(threshold) )
    fstr.write('relocate=1 \n')
    fstr.write("cornersfrom=\'{}\'\n".format(corners_from))
    fstr.write('\n')


def run_fitter(polygon, preset_reso): 
# https://stackoverflow.com/
# questions/12605498/how-to-use-subprocess-popen-python
    temp_dir = TEMP_DIR
    # print( glob.glob( TEMP_DIR + '*' ) )
    try:    os.remove( glob.glob( TEMP_DIR + '*' ) )
    except: pass
    
    exe_dir = None
    if( platform.system() == 'Linux'):
        exe_dir = EXE_DIR_UBUNTU
    else:
        exe_dir = EXE_DIR_WIN
        
    input_pot_dir = '{}/in.pot'.format(temp_dir)
    input_poly_dir = '{}/in.poly'.format(temp_dir)
    output_dir = '{}/out'.format(temp_dir)
    output_reg_dir = '{}_result_step0_black.svg'.format(output_dir)

    if not os.path.exists(temp_dir):
        os.makedirs(temp_dir)
    assert os.path.exists(exe_dir), \
        "Could not find fitter executable {}".format(exe_dir)

    # write the polygon
    polygon.write_dot_poly(input_poly_dir)

    options_dict = get_preset(preset_reso) #for now
    threshold = -0.75
    # https://stackoverflow.com/questions/9184107/how-can-i-force-pythons-file-write-to-use-the-same-newline-format-in-windows
    # on python 2.7 use open(input_dir , 'wb')
    with open(input_pot_dir , 'w', newline='\n') as f:
        write_pot_file(f, input_poly_dir, output_dir, options_dict, threshold, use_proba=True)
    
    process = subprocess.Popen([exe_dir, '-i', input_pot_dir],
                               stderr=subprocess.STDOUT)
    process.communicate()

        
    return output_reg_dir


def run_fitter_multi(preset_reso):
    options_dict = get_preset(preset_reso)
    threshold = -0.75

    input_pot_dir = TEMP_DIR+"in.pot"
    output_path = '%s/out'%TEMP_DIR

    if( platform.system() == 'Linux'):
        exe_dir = 'Executable not present on this platform'
    else:
        exe_dir = MULTIEXE_DIR_WIN

    output_svg = '%s_result-color.svg'%(output_path)
    output_bw_svg = '%s_result-black.svg'%(output_path)

    with open(input_pot_dir, 'w', newline='\n') as ff:
        write_pot_file(ff, '', output_path, options_dict, threshold, use_proba=True,  multirepo=MULTIPOLYGON_DIR)

    #print('Executing %s -i %s'%(exe_dir, input_pot_dir))
    process = subprocess.Popen([exe_dir, '-i', input_pot_dir], shell=True, stderr=subprocess.STDOUT)
    process.communicate()

    # cmd = '"%s" -i %s'%(exe_dir, input_pot_dir)
    # print(cmd)
    # os.system(cmd)

    return output_bw_svg, output_svg
