#!/usr/bin/env python3



"""line_by_line_reader.py

Holds the object LineByLineReader which I have also written in C++

"""

import ast

######################################################################
##                          LineByLineReader
######################################################################
class LineByLineReader:

    ##--------------------- Constructor
    def __init__(self,filename):
        self.n_line = 0
        self.n_col = 0
        self.cols = []
        self.filename = filename
        file = open(filename, "r")
        self.lines = file.readlines()
        file.close()

    ##---------------  Get the next line 
    def read_line(self):
        success = 0
        self.n_col = 0
        
        # get the first uncommented line
        for line in self.lines[self.n_line:]:
            line = line.split('#', 1)[0]
            line = line.rstrip()
            self.n_line = self.n_line + 1
            if line == '':
                continue
            success = 1
            self.cols = line.split()
            return True

        if (success == 0):
            self.cols = []
            return False
        
    ##--------------------- Read a single token
    def read_token(self, convert_to_t):
        ## Check that we don't exceed the columns
        if (self.n_col >= len( self.cols ) ):
            raise StandardError(
                " Trying to access file " +
                " ``%s'' line %d, column %d, total cols %d" %
                (self.filename, self.n_line, self.n_col, len(self.cols))
            )

        ## return the value
        try:
            out =  convert_to_t( self.cols[ self.n_col ] )
        except Exception as err:
            message = ( "Could not read %s line %d col %d to desired type" %
                   (self.filename, self.n_line, self.n_col) )
            raise type(err)(err.message + '\n %s' % message)

        ## Increase counter
        self.n_col = self.n_col+1
        return out

    ##--------------------- Read a single token
    def read_tokens_as_string(self):

        if (self.n_col >= len( self.cols ) ):
            raise StandardError(
                " Trying to access [%s] line %d column %d, total cols %d" %
                (self.filename, self.n_line, self.n_col, len(self.cols))
            )
        
        out = ''
        while (self.n_col < len(self.cols) ):
            out = out + ' ' + self.read_token(str)
        return out.strip()

    ##--------------------- Read a sandwich (dict, array, ...)
    def read_sandwich(self,loafl,loafr):

        ## Keep reading lines until you reach ``{''
        strout = self.read_tokens_as_string()
        while (strout[0] !=  loafl):
            if( not self.read_line() ): return None
            strout = self.read_tokens_as_string()

        ## Read until you reach the last line
        while (strout[-1] != loafr):
            self.read_line()
            line = self.read_tokens_as_string()
            strout = strout + ' ' + line

        ## Return the result
        return ast.literal_eval(strout)
        ##return eval(strout)
