#!/usr/bin/env python3

from PyQt4 import QtGui, QtSvg, QtCore
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg

import re

class SvgWidget(QtSvg.QSvgWidget):
    def __init__(self, fname, parent=None):
        super().__init__(fname, parent)
        br = QtSvg.QGraphicsSvgItem(fname).boundingRect()
        self.svg_height = br.height()
        self.svg_width = br.width()

        viewbox_groups = None
        pattern = r'viewBox="([\d-]+) ([\d-]+) ([\d-]+) ([\d-]+)"'
        with open(fname) as f:
            for i, line in enumerate(f):
                for match in re.finditer(pattern, line):
                    viewbox_groups = match.groups()

        self.svg_width = float(viewbox_groups[3]) 
        self.svg_height = float(viewbox_groups[2]) 

    
    def paintEvent(self, paint_event):
        painter = QtGui.QPainter(self)
        # default_width  = self.renderer().defaultSize().width()
        # default_height = self.renderer().defaultSize().height()
        default_width  = self.svg_height
        default_height = self.svg_width
        widget_width  = self.size().width()
        widget_height = self.size().height()
        ratio_x = widget_width / default_width
        ratio_y = widget_height / default_height
        if ratio_x < ratio_y:
            new_width = widget_width
            new_height = widget_width * default_height / default_width
            new_left = 0
            new_top =  (widget_height - new_height) / 2
        else:
            new_width = widget_height * default_width / default_height
            new_height = widget_height
            new_left = (widget_width - new_width) / 2
            new_top = 0
        self.renderer().render( painter, QtCore.QRectF(new_left, new_top, new_width, new_height))



class CanvasWidget(QtGui.QWidget):
    """A place to draw your stuff, either using painter or matplotlib."""
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setLayout( QtGui.QHBoxLayout(self) )
        self._svg_widget = None
        self._mpl_widget = None

    def is_active(self):
        if(     (self._svg_widget is not None)
            or  (self._svg_widget is not None)):
            return True
        else:
            return False
        
    def clear_figure(self):
        if( self._svg_widget ):
            self.layout().removeWidget( self._svg_widget )
            self._svg_widget.deleteLater()
            self._svg_widget = None
        if( self._mpl_widget ):
            self.layout().removeWidget( self._mpl_widget )
            self._mpl_widget.deleteLater()
            self._mpl_widget = None
        
    def show_mpl(self, fig):
        self.clear_figure()
        self._mpl_widget = mpl = FigureCanvasQTAgg(fig)
        mpl.setSizePolicy( QtGui.QSizePolicy.Expanding,
                           QtGui.QSizePolicy.Expanding)
        mpl.updateGeometry()
        self.layout().addWidget( mpl )

    def show_svg(self, fname):
        self.clear_figure()
        self._svg_widget = SvgWidget(fname)
        self.layout().addWidget( self._svg_widget )

## ====================================================
##                     Test code
## ====================================================
# Create an instance of the application window and run it
def run_min_example():
    import sys
    import numpy as np
    import matplotlib.pyplot as plt
    
    qt_app = QtGui.QApplication(sys.argv)

    cw = CanvasWidget()
    # cw.show_svg('example/Zeichen_123.svg')
    cw.show_svg('temp/out_result_regularizer_step4_black.svg')
    cw.show()

    cw2 = CanvasWidget()
    plt.figure()
    x = np.linspace(1,10,40)
    y = np.sin(x)
    plt.plot(x,y,'-r')
    cw2.show_mpl(plt.gcf())
    cw2.show()
    
    sys.exit(qt_app.exec_())

if __name__ == '__main__':
    run_min_example()
